@@ -1,13 +1,12 @@
 //! A dictionary check with affixes, backed by `libhunspell`
 //!
-//! Does not check grammar, but tokenizes the documentation chunk,
-//! and checks the individual tokens against the dictionary using
-//! the defined affixes.
-//! Can handle multiple dictionaries.
+//! Does not check grammar, but tokenizes the documentation chunk, and checks
+//! the individual tokens against the dictionary using the defined affixes. Can
+//! handle multiple dictionaries.
 
-use super::{apply_tokenizer, Checker, Detector, Documentation, Suggestion, SuggestionSet};
+use super::{apply_tokenizer, Checker, Detector, Suggestion};
 
-use crate::config::Lang5;
+use crate::config::{Lang5, WrappedRegex};
 use crate::documentation::{CheckableChunk, ContentOrigin, PlainOverlay};
 use crate::util::sub_chars;
 use crate::Range;
@@ -16,8 +15,9 @@ use fs_err as fs;
 use io::Write;
 use lazy_static::lazy_static;
 use log::{debug, trace};
-use rayon::prelude::*;
+use nlprule::Tokenizer;
 use std::io::{self, BufRead};
+
 use std::path::{Path, PathBuf};
 use std::sync::Arc;
 
@@ -29,15 +29,6 @@ use super::quirks::{
     replacements_contain_dashed, replacements_contain_dashless, transform, Transformed,
 };
 
-pub struct HunspellWrapper(pub Arc<Hunspell>);
-
-// This is ok, we make sure the state is only run
-// once.
-// TODO remove this and make things explicitly typed
-// TODO this is rather insane.
-unsafe impl Send for HunspellWrapper {}
-unsafe impl Sync for HunspellWrapper {}
-
 static BUILTIN_HUNSPELL_AFF: &[u8] = include_bytes!(concat!(
     env!("CARGO_MANIFEST_DIR"),
     "/hunspell-data/en_US.aff"
@@ -100,8 +91,8 @@ fn cache_builtin() -> Result<(PathBuf, PathBuf)> {
     Ok((path_dic, path_aff))
 }
 
-/// The value is `true` if string is made of emoji's
-/// or Unicode `VULGAR FRACTION`.
+/// The value is `true` if string is made of emoji's or Unicode
+/// `VULGAR FRACTION`.
 pub fn consists_of_vulgar_fractions_or_emojis(word: &str) -> bool {
     lazy_static! {
         static ref VULGAR_OR_EMOJI: regex::RegexSet = regex::RegexSetBuilder::new(&[
@@ -115,10 +106,60 @@ pub fn consists_of_vulgar_fractions_or_emojis(word: &str) -> bool {
     return VULGAR_OR_EMOJI.is_match(word);
 }
 
-pub struct HunspellChecker;
+#[derive(Clone)]
+struct HunspellSafe(Arc<Hunspell>);
 
-impl HunspellChecker {
-    fn inner_init(config: &<Self as Checker>::Config) -> Result<HunspellWrapper> {
+unsafe impl Send for HunspellSafe {}
+// We only use it in RO so it's ok.
+unsafe impl Sync for HunspellSafe {}
+
+impl std::ops::Deref for HunspellSafe {
+    type Target = Hunspell;
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
+impl From<Hunspell> for HunspellSafe {
+    fn from(hunspell: Hunspell) -> Self {
+        Self(Arc::new(hunspell))
+    }
+}
+
+#[derive(Clone)]
+pub struct HunspellCheckerInner {
+    hunspell: HunspellSafe,
+    transform_regex: Vec<WrappedRegex>,
+    allow_concatenated: bool,
+    allow_dashed: bool,
+    allow_emojis: bool,
+    ignorelist: String,
+}
+
+impl HunspellCheckerInner {
+    fn new(config: &<HunspellChecker as Checker>::Config) -> Result<Self> {
+        // TODO allow override
+        let (transform_regex, allow_concatenated, allow_dashed, allow_emojis) = {
+            let quirks = &config.quirks;
+            {
+                (
+                    quirks.transform_regex().to_vec(),
+                    quirks.allow_concatenated(),
+                    quirks.allow_dashed(),
+                    quirks.allow_emojis(),
+                )
+            }
+        };
+        // FIXME rename the config option
+        let ignorelist = config.tokenization_splitchars.clone();
+        // without these, a lot of those would be flagged as mistakes.
+        debug_assert!(ignorelist.contains(','));
+        debug_assert!(ignorelist.contains('.'));
+        debug_assert!(ignorelist.contains(';'));
+        debug_assert!(ignorelist.contains('!'));
+        debug_assert!(ignorelist.contains('?'));
+
+        // setup hunspell:
         let search_dirs = config.search_dirs();
 
         let lang = config.lang().to_string();
@@ -213,7 +254,33 @@ impl HunspellChecker {
             }
         }
         debug!("Dictionary setup completed successfully.");
-        Ok(HunspellWrapper(Arc::new(hunspell)))
+        Ok(Self {
+            hunspell: HunspellSafe::from(hunspell),
+            transform_regex,
+            allow_concatenated,
+            allow_dashed,
+            allow_emojis,
+            ignorelist,
+        })
+    }
+}
+
+#[derive(Clone)]
+pub struct HunspellChecker(pub Arc<HunspellCheckerInner>, pub Arc<Tokenizer>);
+
+impl std::ops::Deref for HunspellChecker {
+    type Target = HunspellCheckerInner;
+    fn deref(&self) -> &Self::Target {
+        self.0.deref()
+    }
+}
+
+impl HunspellChecker {
+    pub fn new(config: &<HunspellChecker as Checker>::Config) -> Result<Self> {
+        let tokenizer = super::tokenizer::<&PathBuf>(None)?;
+        let inner = HunspellCheckerInner::new(config)?;
+        let hunspell = Arc::new(inner);
+        Ok(HunspellChecker(hunspell, tokenizer))
     }
 }
 
@@ -224,123 +291,84 @@ impl Checker for HunspellChecker {
         Detector::Hunspell
     }
 
-    fn check<'a, 's>(docu: &'a Documentation, config: &Self::Config) -> Result<SuggestionSet<'s>>
+    fn check<'a, 's>(
+        &self,
+        origin: &ContentOrigin,
+        chunks: &'a [CheckableChunk],
+    ) -> Result<Vec<Suggestion<'s>>>
     where
         'a: 's,
     {
-        let hunspell = Self::inner_init(config)?;
+        let mut acc = Vec::with_capacity(chunks.len());
 
-        let (transform_regex, allow_concatenated, allow_dashed, allow_emojis) = {
-            let quirks = &config.quirks;
-            {
-                (
-                    quirks.transform_regex(),
-                    quirks.allow_concatenated(),
-                    quirks.allow_dashed(),
-                    quirks.allow_emojis(),
-                )
-            }
-        };
-        // FIXME rename the config option
-        let ignorelist = config.tokenization_splitchars.as_str();
-        // without these, a lot of those would be flagged as mistakes.
-        debug_assert!(ignorelist.contains(','));
-        debug_assert!(ignorelist.contains('.'));
-        debug_assert!(ignorelist.contains(';'));
-        debug_assert!(ignorelist.contains('!'));
-        debug_assert!(ignorelist.contains('?'));
+        for chunk in chunks {
+            let plain = chunk.erase_cmark();
+            trace!("{:?}", &plain);
+            let txt = plain.as_str();
+            let hunspell = &*self.hunspell.0;
 
-        // TODO allow override
-        let tokenizer = super::tokenizer::<&PathBuf>(None)?;
-
-        let suggestions = docu
-            .par_iter()
-            .try_fold::<SuggestionSet, Result<_>, _, _>(
-                || SuggestionSet::new(),
-                move |mut acc, (origin, chunks)| {
-                    debug!("Processing {}", origin.as_path().display());
-
-                    for chunk in chunks {
-                        let plain = chunk.erase_cmark();
-                        trace!("{:?}", &plain);
-                        let txt = plain.as_str();
-                        let hunspell = &*hunspell.0;
-
-                        'tokenization: for range in apply_tokenizer(&tokenizer, txt) {
-                            let word = sub_chars(txt, range.clone());
-                            if range.len() == 1
-                                && word
-                                    .chars()
-                                    .next()
-                                    .filter(|c| ignorelist.contains(*c))
-                                    .is_some()
-                            {
-                                continue 'tokenization;
-                            }
-                            if transform_regex.is_empty() {
+            'tokenization: for range in apply_tokenizer(&self.1, txt) {
+                let word = sub_chars(txt, range.clone());
+                if range.len() == 1
+                    && word
+                        .chars()
+                        .next()
+                        .filter(|c| self.ignorelist.contains(*c))
+                        .is_some()
+                {
+                    continue 'tokenization;
+                }
+                if self.transform_regex.is_empty() {
+                    obtain_suggestions(
+                        &plain,
+                        chunk,
+                        &hunspell,
+                        &origin,
+                        word,
+                        range,
+                        self.allow_concatenated,
+                        self.allow_dashed,
+                        self.allow_emojis,
+                        &mut acc,
+                    )
+                } else {
+                    match transform(&self.transform_regex[..], word.as_str(), range.clone()) {
+                        Transformed::Fragments(word_fragments) => {
+                            for (range, word_fragment) in word_fragments {
                                 obtain_suggestions(
                                     &plain,
                                     chunk,
                                     &hunspell,
-                                    origin,
-                                    word,
+                                    &origin,
+                                    word_fragment.to_owned(),
                                     range,
-                                    allow_concatenated,
-                                    allow_dashed,
-                                    allow_emojis,
+                                    self.allow_concatenated,
+                                    self.allow_dashed,
+                                    self.allow_emojis,
                                     &mut acc,
-                                )
-                            } else {
-                                match transform(&transform_regex[..], word.as_str(), range.clone())
-                                {
-                                    Transformed::Fragments(word_fragments) => {
-                                        for (range, word_fragment) in word_fragments {
-                                            obtain_suggestions(
-                                                &plain,
-                                                chunk,
-                                                &hunspell,
-                                                origin,
-                                                word_fragment.to_owned(),
-                                                range,
-                                                allow_concatenated,
-                                                allow_dashed,
-                                                allow_emojis,
-                                                &mut acc,
-                                            );
-                                        }
-                                    }
-                                    Transformed::Atomic((range, word)) => {
-                                        obtain_suggestions(
-                                            &plain,
-                                            chunk,
-                                            &hunspell,
-                                            origin,
-                                            word.to_owned(),
-                                            range,
-                                            allow_concatenated,
-                                            allow_dashed,
-                                            allow_emojis,
-                                            &mut acc,
-                                        );
-                                    }
-                                    Transformed::Whitelisted(_) => {}
-                                }
+                                );
                             }
                         }
+                        Transformed::Atomic((range, word)) => {
+                            obtain_suggestions(
+                                &plain,
+                                chunk,
+                                &hunspell,
+                                &origin,
+                                word.to_owned(),
+                                range,
+                                self.allow_concatenated,
+                                self.allow_dashed,
+                                self.allow_emojis,
+                                &mut acc,
+                            );
+                        }
+                        Transformed::Whitelisted(_) => {}
                     }
-                    Ok(acc)
-                },
-            )
-            .try_reduce(
-                || SuggestionSet::new(),
-                |mut a, b| {
-                    a.join(b);
-                    Ok(a)
-                },
-            )?;
-
-        // TODO sort spans by file and line + column
-        Ok(suggestions)
+                }
+            }
+        }
+        Ok(acc)
     }
 }
 
@@ -354,7 +382,7 @@ fn obtain_suggestions<'s>(
     allow_concatenated: bool,
     allow_dashed: bool,
     allow_emojis: bool,
-    acc: &mut SuggestionSet<'s>,
+    acc: &mut Vec<Suggestion<'s>>,
 ) {
     if !hunspell.check(&word) {
         trace!("No match for word (plain range: {:?}): >{}<", &range, &word);
@@ -380,18 +408,15 @@ fn obtain_suggestions<'s>(
             return;
         }
         for (range, span) in plain.find_spans(range.clone()) {
-            acc.add(
-                origin.clone(),
-                Suggestion {
-                    detector: Detector::Hunspell,
-                    range,
-                    span,
-                    origin: origin.clone(),
-                    replacements: replacements.clone(),
-                    chunk,
-                    description: Some("Possible spelling mistake found.".to_owned()),
-                },
-            )
+            acc.push(Suggestion {
+                detector: Detector::Hunspell,
+                range,
+                span,
+                origin: origin.clone(),
+                replacements: replacements.clone(),
+                chunk,
+                description: Some("Possible spelling mistake found.".to_owned()),
+            })
         }
     } else {
         trace!(
