@@ -2,8 +2,8 @@
 //!
 //! Supports `Hunspell` and `LanguageTool` scopes.
 //!
-//! A default configuration will be generated in the default
-//! location by default. Default. Default default default.
+//! A default configuration will be generated in the default location by
+//! default. Default. Default default default.
 
 // TODO pendeng refactor, avoid spending time on documenting the status quo.
 #![allow(missing_docs)]
@@ -85,8 +85,7 @@ impl Config {
     const ORGANIZATION: &'static str = "spearow";
     const APPLICATION: &'static str = "cargo_spellcheck";
 
-    /// Sanitize all relative paths to absolute paths
-    /// in relation to `base`.
+    /// Sanitize all relative paths to absolute paths in relation to `base`.
     fn sanitize_paths(&mut self, base: &Path) -> Result<()> {
         if let Some(ref mut hunspell) = self.hunspell {
             hunspell.sanitize_paths(base)?;
@@ -260,6 +259,13 @@ impl Default for Config {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use assert_matches::assert_matches;
+
+    #[test]
+    fn can_serialize_to_toml() {
+        let config = dbg!(Config::full());
+        assert_matches!(config.to_toml(), Ok(_s));
+    }
 
     #[test]
     fn all() {
