@@ -64,8 +64,8 @@ fn parse_and_construct() {
 
 use crate::documentation::Documentation;
 
-/// Declare an end-to-end test case, ranging from input content
-/// down to the number expected issues given a checker type.
+/// Declare an end-to-end test case, ranging from input content down to the
+/// number expected issues given a checker type.
 #[macro_export]
 /// End-to-end tests for different Checkers
 macro_rules! end2end {
@@ -74,6 +74,11 @@ macro_rules! end2end {
     }};
 
     ($test:expr, $origin:expr, $n:expr, $checker:ty) => {{
+        let cfg = dbg!(Default::default());
+        end2end!($test, $origin, $n, $checker, cfg);
+    }};
+
+    ($test:expr, $origin:expr, $n:expr, $checker:ty, $cfg:expr) => {{
         let _ = env_logger::builder()
             .is_test(true)
             .filter_level(log::LevelFilter::Trace)
@@ -86,15 +91,12 @@ macro_rules! end2end {
         assert_eq!(dbg!(chunks).len(), 1);
         let chunk = &chunks[0];
         let _plain = chunk.erase_cmark();
-
-        let cfg = dbg!(Default::default());
+        let cfg = $cfg;
         dbg!(std::any::type_name::<$checker>());
-        let suggestion_set = <$checker as Checker>::check(&docs, &cfg)
+        let checker = <$checker>::new(&cfg).expect("Checker construction works");
+        let suggestions = checker
+            .check(&origin, &chunks[..])
             .expect("Must not fail to extract suggestions");
-        let (_, suggestions) = dbg!(&suggestion_set)
-            .iter()
-            .next()
-            .expect("Suggestion set must not be empty");
         assert_eq!(suggestions.len(), $n);
     }};
 }
@@ -114,6 +116,7 @@ macro_rules! end2end_file_rust {
 }
 
 /// Declare an end-to-end test case based on an existing common mark file.
+#[allow(unused_macros)]
 macro_rules! end2end_file_cmark {
     ($path: literal, $n: expr) => {{
         let path2 = PathBuf::from(concat!(env!("CARGO_MANIFEST_DIR"), "/", $path));
@@ -150,6 +153,60 @@ mod e2e {
         end2end!(chyrp_up!(["Alphy", "Beto"]), 2);
     }
 
+    use crate::checker::HunspellChecker;
+
+    #[test]
+    fn issue_226() {
+        use crate::config::*;
+        use fancy_regex::Regex;
+
+        let transform_regex = [r#"\\\[()?:[1-9][0-9]*\\\]"#]
+            .iter()
+            .map(|&x| WrappedRegex(Regex::new(x).unwrap()))
+            .collect::<Vec<_>>();
+
+        let cfg = crate::config::HunspellConfig {
+            // FIXME splitchars
+            quirks: crate::config::Quirks {
+                transform_regex,
+                ..Default::default()
+            },
+            ..Default::default()
+        };
+
+        end2end!(
+            r####"
+/// X is [\[1790\]]
+///
+/// [\[1790\]]: https://ahoi.io
+struct X;
+            "####,
+            ContentOrigin::TestEntityRust,
+            0,
+            HunspellChecker,
+            cfg
+        );
+    }
+
+    #[test]
+    fn issue_227() {
+        // The test
+        end2end!(
+            r####"
+/// ```
+/// use std::path::PathBuf as A;
+#[doc = "// Hello"]
+/// use std::path::PathBuf as B;
+/// ```
+struct X;
+            "####,
+            ContentOrigin::TestEntityRust,
+            0,
+            DummyChecker,
+            Default::default()
+        );
+    }
+
     #[test]
     fn file_justone() {
         end2end_file_rust!("demo/src/nested/justone.rs", 2);
@@ -177,14 +234,11 @@ mod e2e {
             let origin: ContentOrigin = $origin;
 
             let docs = Documentation::load_from_str(origin.clone(), $source, false);
-            let suggestion_set =
-                dbg!(DummyChecker::check(&docs, &())).expect("Dummy checker never fails. qed");
+            let (origin2, chunks) = docs.into_iter().next().expect("Contains a document");
+            let suggestions =
+                dbg!(DummyChecker.check(&origin, &chunks[..])).expect("Dummy checker never fails. qed");
 
-            let (origin2, chunks) = docs
-                .iter()
-                .next()
-                .expect("Introduced exactly one source. qed");
-            assert_eq!(&origin, origin2);
+            assert_eq!(origin, origin2);
 
             let chunk = chunks
                 .iter()
@@ -197,9 +251,6 @@ mod e2e {
             let plain = dbg!(chunk.erase_cmark());
             assert_eq!(dbg!($plain), plain.as_str());
 
-            let mut it = suggestion_set.iter();
-            let (_, suggestions) = it.next().expect("Dummy checker produces one error per tokenized word. qed");
-
             let mut it = suggestions.into_iter();
 
             let mut expected = |word: &'static str| {
