@@ -1,7 +1,7 @@
 use super::*;
 use crate::util::load_span_from;
 use crate::{chyrp_up, fluff_up};
-use crate::{LineColumn, Span};
+use crate::{Documentation, LineColumn, Span};
 
 macro_rules! verify_reflow_inner {
     ($n:literal break [ $( $line:literal ),+ $(,)?] => $expected:literal) => {
@@ -199,9 +199,9 @@ macro_rules! reflow_content {
     };
 }
 
-/// Run reflow on a set of lines that are `fluff_up`ed
-/// and match the resulting `Patch`s replacement with
-/// an `expected` (a single literal, TODO allow multiple).
+/// Run reflow on a set of lines that are `fluff_up`ed and match the resulting
+/// `Patch`s replacement with an `expected` (a single literal, TODO allow
+/// multiple).
 macro_rules! reflow_fluff {
     ($n:literal break [ $( $line:literal ),+ $(,)?] => $( $expected:literal ),+ $(,)?) => {
         const CONTENT:&'static str = fluff_up!($( $line ),+);
